@@ -19,12 +19,14 @@
 #' @seealso \code{\link{mselect}}, \code{\link{getDim}}
 #' @examples
 #' 
-#' data(population_magpie)
-#' dimCode(c("t","scenario","blablub"),population_magpie)
+#' pop <- maxample("pop")
+#' dimCode(c("t","scenario","blablub"),pop)
 #' 
 #' @export dimCode
 dimCode <- function(dim, x, missing=0, sep="."){
   
+  if(all(is.character(dim)) && any(grepl(sep, dim, fixed=TRUE))) stop("Dimension separator must not be used in dimension name!")
+  
   #function to translate dim to dim code
   if(is.character(dim)) {
     
