@@ -77,7 +77,11 @@ magpie_expand_dim <- function(x,ref,dim=1) {
     }
   }
   
-  m <- merge(dref,dx,sort=FALSE, suffixes=c("_ref","_x"), by=setdiff(intersect(names(dref), names(dx)),".line"))
+  m <- merge(dref,dx,sort=FALSE, suffixes=c("_ref","_x"), 
+             by=setdiff(intersect(names(dref), names(dx)),".line"),
+             all.x=TRUE, all.y=TRUE)
+  
+  if(anyNA(c(m$.line_ref,m$.line_x))) stop(" Identical set names but different content. Correct set names!")
   
   # reorder m so that dref columns appear first
   m <- m[union(names(dref)[1:(ncol(dref)-1)],names(m))]
