@@ -22,8 +22,7 @@ m3 <- as.magpie(v, spatial=0)
 str(m3)
 
 ## ---- echo = TRUE-------------------------------------------------------------
-data("population_magpie")
-pm <- population_magpie
+pm <- maxample("pop")
 
 ## ---- echo = TRUE-------------------------------------------------------------
 str(pm)
@@ -86,6 +85,9 @@ pm["AS",,,pmatch=TRUE]
 ## ---- echo = TRUE-------------------------------------------------------------
 mselect(pm, scenario="B1", i=c("FSU","LAM"))
 
+## ---- echo = TRUE-------------------------------------------------------------
+pm[list(i=c("FSU","LAM")),,list(scenario="B1")]
+
 ## ---- echo = TRUE-------------------------------------------------------------
 d <- head(pm)
 
